package sample.parser;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.lightysoft.logmx.business.ParsedEntry;
import com.lightysoft.logmx.mgr.LogFileParser;

/**
 * LogMX Parser able to parse a log file with a giant single JSON string on a single line, describing multiple log entries.<BR/>
 * Here is an example of log file suitable for this parser:<BR/>
 * 
 *   [{"d":"Thu, 12 Mar 2020 16:29:49 GMT","t":"WARN","m":"[VidyoMediaManager] The sococo camera limit is 4 "},{"d":"Thu, 12 Mar 2020 16:29:49 GMT","t":"WARN","m":"[Vidyo] service initialized "},{"d":"Thu, 12 Mar 2020 16:29:50 GMT","t":"WARN","m":"[vidyoplugin] presence connected: firstload: true, gated: false, failed: false "},{"d":"Thu, 12 Mar 2020 16:31:43 GMT","t":"WARN","m":"$health(Media Connection) is -> Failed ✖"},{"d":"Thu, 12 Mar 2020 16:31:43 GMT","t":"WARN","m":"$health(Media Connection) is repaired ✓"},{"d":"Thu, 12 Mar 2020 16:32:36 GMT","t":"WARN","m":"$health(Media Connection) is -> Failed ✖"}]
 */
public class SococoJsonLogsParser extends LogFileParser {
    /** Entry date formatter */
    private SimpleDateFormat dateFormat = null;

    /** Entry date format string */
    private final String DATE_FORMAT_STRING = "EEE, dd MMM yyyy HH:mm:ss z"; // TODO: make sure the used abbreviations for the Day and Month are compatible with this Java formatter

    /** Mutex to avoid that multiple threads use the same Date formatter at the same time */
    private final Object DATE_FORMATTER_MUTEX = new Object();

    /** Number of time this parser fails to parse a line as JSON */
    private int numParsingFailures = 0;


    /** 
     * Returns the name of this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getParserName()
     */
    @Override
    public String getParserName() {
        return "Sococo JSON Logs Parser";
    }

    /**
     * Returns the supported file type for this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getSupportedFileType()
     */
    @Override
    public String getSupportedFileType() {
        return "Sococo JSON log files";
    }

    /**
     * Process the new line of text read from file 
     * @see com.lightysoft.logmx.mgr.LogFileParser#parseLine(java.lang.String)
     */
    @Override
    protected void parseLine(String line) throws Exception {
        if (line == null || numParsingFailures >= 3) { // do not try to parse too many lines as JSON to avoid a performance hit when LogMX tries to detect the format of other log formats
            return;
        }

        try {
            JsonElement jsonElt = JsonParser.parseString(line);
            if (jsonElt instanceof JsonObject) {
                processLogEntryJsonObject((JsonObject) jsonElt);
            } else if (jsonElt instanceof JsonArray) {
                JsonArray jsonArray = (JsonArray) jsonElt;
                for (Iterator<JsonElement> it = jsonArray.iterator(); it.hasNext();) {
                    JsonElement arrayElt = it.next();
                    if (arrayElt instanceof JsonObject) {
                        processLogEntryJsonObject((JsonObject) arrayElt);
                    }
                }
            }

        } catch (Exception ex) {
            // Not a JSON string
            numParsingFailures++;
        }
    }

    private void processLogEntryJsonObject(JsonObject pJsonObj) throws Exception {
        ParsedEntry entry = createNewEntry();
        JsonElement dateElt = pJsonObj.get("d");
        JsonElement levelElt = pJsonObj.get("t");
        JsonElement msgElt = pJsonObj.get("m");
        entry.setDate(dateElt == null ? null : dateElt.getAsString());
        entry.setLevel(levelElt == null ? null : levelElt.getAsString());
        entry.setMessage(msgElt == null ? null : msgElt.getAsString());
        addEntry(entry);
    }

    /**
     * Returns a relative Date for the given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getRelativeEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getRelativeEntryDate(ParsedEntry pEntry) throws Exception {
        return getAbsoluteEntryDate(pEntry);
    }

    /**
     * Returns the absolute Date for the given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getAbsoluteEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getAbsoluteEntryDate(ParsedEntry pEntry) throws Exception {
        if (dateFormat == null) {
            // Now create the date formatter using the right Locale
            // (method "getLocale()" can't be called from the constructor) 
            dateFormat = new SimpleDateFormat(DATE_FORMAT_STRING, getLocale());
        }
        synchronized (DATE_FORMATTER_MUTEX) { // Java date formatter is not thread-safe
            return dateFormat.parse(pEntry.getDate());
        }
    }
}
